@@ -1,7 +1,7 @@
 /*
  * Plumbing.c
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd and Others
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd and Others
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -16,10 +16,10 @@
 
 /*
  * Implements
- *   org.freedesktop.bindings.Plumbing.createProxy(Class type, long pointer)
+ *   org.freedesktop.bindings.Plumbing.createPointer(Class type, long pointer)
  */
 JNIEXPORT jobject JNICALL
-Java_org_freedesktop_bindings_Plumbing_createProxy
+Java_org_freedesktop_bindings_Plumbing_createPointer
 (
 	JNIEnv *env,
 	jclass cls,
@@ -32,7 +32,7 @@ Java_org_freedesktop_bindings_Plumbing_createProxy
 	
 	constructor = (*env)->GetMethodID(env, type, "<init>", "(J)V");
 	if (constructor == NULL) {
-		g_error("Constructor methodID not found");
+		g_critical("Constructor methodID not found");
 		return NULL;
 	}
 	
@@ -59,10 +59,30 @@ Java_org_freedesktop_bindings_Plumbing_createFlag
 	
 	constructor = (*env)->GetMethodID(env, type, "<init>", "(ILjava/lang/String;)V");
 	if (constructor == NULL) {
-		g_error("Constructor methodID not found");
+		g_critical("Constructor methodID not found");
 		return NULL;
 	}
 	
 	flag = (*env)->NewObject(env, type, constructor, ordinal, nickname);
 	return flag;
 }
+
+
+/*
+ * Implements
+ *   org.freedesktop.bindings.Plumbing.toHexString(long pointer)
+ */
+JNIEXPORT jstring JNICALL
+Java_org_freedesktop_bindings_Plumbing_toHexString
+(
+	JNIEnv *env,
+	jclass cls,
+	jlong _pointer
+)
+{
+	const gchar* result;
+	
+	result = bindings_java_memory_pointerToString((gpointer) _pointer);
+	
+	return (*env)->NewStringUTF(env, result);
+}
