@@ -1,7 +1,7 @@
 /*
  * GdkPixbufOverride.java
  *
- * Copyright (c) 2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2008-2009 Operational Dynamics Consulting Pty Ltd
  *
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -11,8 +11,12 @@
  */
 package org.gnome.gdk;
 
+import org.gnome.glib.GlibException;
+
 final class GdkPixbufOverride extends Plumbing
 {
+    private GdkPixbufOverride() {}
+
     static final byte[] getPixels(Pixbuf self) {
         synchronized (lock) {
             return gdk_pixbuf_get_pixels(pointerOf(self));
@@ -20,4 +24,16 @@ final class GdkPixbufOverride extends Plumbing
     }
 
     private static native final byte[] gdk_pixbuf_get_pixels(long self);
+
+    static final long createPixbufFromArray(byte[] data) throws GlibException {
+        if (data == null) {
+            throw new IllegalArgumentException("byte array can't be null");
+        }
+
+        synchronized (lock) {
+            return gdk_pixbuf_new_from_stream(data);
+        }
+    }
+
+    private static native final long gdk_pixbuf_new_from_stream(byte[] data) throws GlibException;
 }
