@@ -1,7 +1,7 @@
 /*
  * TreePath.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
  *
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -11,16 +11,9 @@
  */
 package org.gnome.gtk;
 
-import org.gnome.glib.Boxed;
+import java.util.StringTokenizer;
 
-/*
- * FIXME this is a placeholder stub for what will become the public API for
- * this type. Replace this comment with appropriate javadoc including author
- * and since tags. Note that the class may need to be made abstract, implement
- * interfaces, or even have its parent changed. No API stability guarantees
- * are made about this class until it has been reviewed by a hacker and this
- * comment has been replaced.
- */
+import org.gnome.glib.Boxed;
 
 /**
  * A logical but abstract representation of a row in a TreeModel. TreePaths
@@ -40,12 +33,12 @@ import org.gnome.glib.Boxed;
  * 
  * <p>
  * TreePaths are given to you as a row address in signals like
- * {@link TreeView.ROW_ACTIVATED ROW_ACTIVATED}. Usually you need the row
- * expressed as a TreeIter (ie to get a value out of the row);to do that call
- * the underlying TreeModel's {@link TreeModel#getIter(TreePath) getIter()}
- * method.
+ * {@link TreeView.RowActivated}. Usually you need the row expressed as a
+ * TreeIter (ie to get a value out of the row);to do that call the underlying
+ * TreeModel's {@link TreeModel#getIter(TreePath) getIter()} method.
  * 
  * @author Andrew Cowie
+ * @author Stefan Prelle
  * @since 4.0.5
  */
 public final class TreePath extends Boxed
@@ -55,7 +48,8 @@ public final class TreePath extends Boxed
     }
 
     /**
-     * Create an empty TreePath object. <b>For use by bindings hackers only!</b>
+     * Create an empty TreePath object. <b>For use by bindings hackers
+     * only!</b>
      */
     TreePath() {
         super(GtkTreePath.createTreePath());
@@ -102,4 +96,56 @@ public final class TreePath extends Boxed
             return false;
         }
     }
+
+    /**
+     * Return the text form represented by this TreePath. For example, a
+     * TreePath pointing at the third row's second child row's fifth child row
+     * in a TreeStore would be "2:1:4"; the 16th row of a ListStore would be
+     * "15".
+     * 
+     * @since 4.0.6
+     */
+    public String toString() {
+        return GtkTreePath.toString(this);
+    }
+
+    /**
+     * Returns the depth of node identified by this TreePath within the tree.
+     * Or with other words, the number of elements in the TreePath.
+     * 
+     * @since 4.0.9
+     */
+    public int getDepth() {
+        return GtkTreePath.getDepth(this);
+    }
+
+    /**
+     * Returns the indices the path consists of as an array of integer. If for
+     * example the path would be "1:4:2" you would get {1,4,2}.
+     * 
+     * @return The indices of the nodes or <code>null</code> if nothing is
+     *         selected.
+     * 
+     * @since 4.0.9
+     */
+    /*
+     * Calling GtkTreePath.getIndices() always returns a null for me, so I
+     * implemented this workaround. If someone willing to dig deeper in the
+     * native code finds out why, this code can be removed.
+     */
+    public int[] getIndices() {
+        final StringTokenizer tok;
+        final int[] ret;
+
+        try {
+            tok = new StringTokenizer(this.toString(), ":");
+            ret = new int[tok.countTokens()];
+            for (int i = 0; i < ret.length; i++) {
+                ret[i] = Integer.parseInt(tok.nextToken());
+            }
+            return ret;
+        } catch (NumberFormatException e) {
+            return null;
+        }
+    }
 }
