@@ -2,7 +2,7 @@
  * bindings_java_signal.c
  *
  * Copyright (c) 1998-2005 The java-gnome Team
- * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -97,13 +97,14 @@ bindings_java_marshaller
 	
 	jstring _str;
 	gchar* str;
+	GObject* obj;
 	
 	/*
 	 * Begin marshaller by downcasting the GClosure we got.
 	 */
 
 	bjc = (BindingsJavaClosure*) closure;
-	
+
 	/*
 	 * Get the JNIEnv interface pointer
 	 */
@@ -131,12 +132,12 @@ bindings_java_marshaller
 	 */
 
 	jargs = g_newa(jvalue, n_param_values + 1);
-	
+
 	jargs[0].l = bjc->handler;
-	
+
 	for(i = 0; i < n_param_values; i++) {
-  		type = G_VALUE_TYPE(&param_values[i]);
-    		switch(G_TYPE_FUNDAMENTAL(type)) {
+		type = G_VALUE_TYPE(&param_values[i]);
+		switch(G_TYPE_FUNDAMENTAL(type)) {
 		case G_TYPE_CHAR:
 			jargs[i+1].c = g_value_get_char(&param_values[i]);
       			break;
@@ -193,8 +194,10 @@ bindings_java_marshaller
 			 * address across the boundary to be looked up and
 			 * either an existing Proxy returned or a new Proxy
 			 * created. 
-			 */			
-			jargs[i+1].j = (jlong) g_value_get_object(&param_values[i]); 
+			 */
+			obj = g_value_get_object(&param_values[i]); 
+			bindings_java_memory_cleanup(obj, FALSE);
+			jargs[i+1].j = (jlong) obj;
 			break;
 
 		case G_TYPE_BOXED:
@@ -206,12 +209,27 @@ bindings_java_marshaller
 			 */
 			jargs[i+1].j = (jlong) g_boxed_copy(type, g_value_get_boxed(&param_values[i]));
 			break;
-			
+
+		case G_TYPE_PARAM:
+			/*
+			 * We're ignoring GParamSpec at the moment. They
+			 * normally only show up in 'notify' signals, and we
+			 * don't need them. 
+			 */
+		case G_TYPE_POINTER:
+			/*
+			 * and, we're ignoring something that gets registered
+			 * as a gpointer, by definition it has no type
+			 * information and there's nothing we can do.
+			 */
+			jargs[i+1].j = (jlong) NULL;
+			break;
+
 		default:
 			/*
 			 * Unrecognized. Probably means we need to add a clause above.
 			 */
-			g_warning("Don't know how to marshal a %s", g_type_name(type));
+			g_printerr("Don't know how to marshal a %s", g_type_name(type));
 			jargs[i+1].l = 0;
 			break;
 		}
@@ -266,7 +284,7 @@ bindings_java_marshaller
 		 * enormous assumption that a string is what we get back.
 		 */
 		_str = (*env)->CallStaticObjectMethodA(env, bjc->receiver, bjc->method, jargs);
-		if (str == NULL) {
+		if (_str == NULL) {
 			g_warning("Invoking string handler returned null. That's probably bad");
 			break;
 		}
@@ -432,7 +450,7 @@ bindings_java_closure_new
       	case G_TYPE_NONE:
       		bjc->returnType = 'V';
       		break;
-	
+
 	default:
 		g_critical("Don't know what to do with signal return type %s", g_type_name(info.return_type));
 		return NULL;
@@ -444,22 +462,28 @@ bindings_java_closure_new
 	 * where Name is a PascalCase version of the signal name we were
 	 * passed in.
 	 */
-	 
+
 	buf = g_string_new("receive");
-	
-	gchar** tokens = g_strsplit_set(name, "_-", -1);
-	
+
+	gchar** tokens = g_strsplit_set(name, "_-:", -1);
+
 	for (i = 0; i < g_strv_length(tokens); i++) {
 		gchar* token = tokens[i];
+
+		if (token[0] == '\0') {
+			// skip past :: which splits signal name from "detail"
+			continue;
+		}
+
 		gchar first = g_unichar_toupper(token[0]);
 		g_string_append_c(buf, first);
-		
+
 		token++; 
 		g_string_append(buf, token);
 	}
-	
+
 	methodName = buf->str;
-	
+
 	g_string_free(buf, FALSE);
 	g_strfreev(tokens);
 
